import { Seg, Slicer, DayTableModel, DateRange, DateProfileGenerator, DateEnv, BaseComponent, ViewProps, DateProfile, DateMarker, DateFormatter, Dictionary, DayTableCell, Hit, EventSegUiInteractionState, DateComponent, RefMap } from '@fullcalendar/core/internal';
import { createElement, Ref, ComponentChild, VNode } from '@fullcalendar/core/preact';
import '@fullcalendar/core';

interface TableSeg extends Seg {
    row: number;
    firstCol: number;
    lastCol: number;
    isStandin?: boolean;
}

declare class DayTableSlicer extends Slicer<TableSeg, [DayTableModel]> {
    forceDayIfListItem: boolean;
    sliceRange(dateRange: DateRange, dayTableModel: DayTableModel): TableSeg[];
}

declare class TableDateProfileGenerator extends DateProfileGenerator {
    buildRenderRange(currentRange: any, currentRangeUnit: any, isRangeAllDay: any): DateRange;
}
declare function buildDayTableRenderRange(props: {
    currentRange: DateRange;
    snapToWeek: boolean;
    fixedWeekCount: boolean;
    dateEnv: DateEnv;
}): DateRange;

declare class DayGridView extends BaseComponent<ViewProps> {
    private buildDayTableModel;
    private buildHeaderTiers;
    private createDayHeaderFormatter;
    private slicer;
    render(): createElement.JSX.Element;
}

interface DateHeaderCellProps {
    dateProfile: DateProfile;
    todayRange: DateRange;
    date: DateMarker;
    navLink: boolean;
    dayHeaderFormat: DateFormatter;
    isSticky?: boolean;
    colSpan?: number;
    extraRenderProps?: Dictionary;
    extraDataAttrs?: Dictionary;
    extraClassNames?: string[];
    colWidth?: number;
    innerHeightRef?: Ref<number>;
}
declare class DateHeaderCell extends BaseComponent<DateHeaderCellProps> {
    private innerElRef;
    private disconectInnerHeight?;
    render(): createElement.JSX.Element;
    componentDidMount(): void;
    componentWillUnmount(): void;
}

interface DayOfWeekHeaderCellProps {
    dow: number;
    dayHeaderFormat: DateFormatter;
    isSticky?: boolean;
    colSpan?: number;
    extraRenderProps?: Dictionary;
    extraDataAttrs?: Dictionary;
    extraClassNames?: string[];
    colWidth?: number;
    innerHeightRef?: Ref<number>;
}
declare class DayOfWeekHeaderCell extends BaseComponent<DayOfWeekHeaderCellProps> {
    private innerElRef;
    private disconectInnerHeight?;
    render(): createElement.JSX.Element;
    componentDidMount(): void;
    componentWillUnmount(): void;
}

interface HeaderRowProps<Model, ModelKey> {
    tierNum: number;
    cells: Model[];
    renderHeaderContent: (model: Model, tier: number, innerHeightRef: Ref<number> | undefined, // unused
    width: number | undefined) => ComponentChild;
    getHeaderModelKey: (model: Model) => ModelKey;
    cellGroup?: boolean;
    className?: string;
    colWidth?: number;
}
declare class HeaderRow<Model, ModelKey> extends BaseComponent<HeaderRowProps<Model, ModelKey>> {
    render(): createElement.JSX.Element;
}

interface HeaderRowAdvancedProps<Model, ModelKey> {
    tierNum: number;
    cells: Model[];
    renderHeaderContent: (// TODO: better name
    model: Model, tier: number, innerHeightRef: Ref<number> | undefined, width: number | undefined) => ComponentChild;
    getHeaderModelKey: (model: Model) => ModelKey;
    innerHeightRef?: Ref<number>;
    height: number | undefined;
    colWidth?: number;
}
declare class HeaderRowAdvanced<Model, ModelKey> extends BaseComponent<HeaderRowAdvancedProps<Model, ModelKey>> {
    private innerHeightRefMap;
    private currentInnerHeight?;
    render(): createElement.JSX.Element;
    private handleInnerHeights;
}

declare function createDayHeaderFormatter(explicitFormat: DateFormatter, datesRepDistinctDays: any, dateCnt: any): DateFormatter;

interface DayGridLayoutProps<HeaderCellModel, HeaderCellKey> {
    dateProfile: DateProfile;
    todayRange: DateRange;
    cellRows: DayTableCell[][];
    forPrint: boolean;
    isHitComboAllowed?: (hit0: Hit, hit1: Hit) => boolean;
    className: string;
    headerTiers: HeaderCellModel[][];
    renderHeaderContent: (model: HeaderCellModel, tier: number, innerHeightRef: Ref<number> | undefined, // unused... why do we have it then???
    width: number | undefined) => ComponentChild;
    getHeaderModelKey: (model: HeaderCellModel) => HeaderCellKey;
    fgEventSegs: TableSeg[];
    bgEventSegs: TableSeg[];
    businessHourSegs: TableSeg[];
    dateSelectionSegs: TableSeg[];
    eventDrag: EventSegUiInteractionState | null;
    eventResize: EventSegUiInteractionState | null;
    eventSelection: string;
}
declare class DayGridLayout<HeaderCellModel, HeaderCellKey> extends BaseComponent<DayGridLayoutProps<HeaderCellModel, HeaderCellKey>> {
    private scrollerRef;
    private rowHeightRefMap;
    render(): createElement.JSX.Element;
    componentDidMount(): void;
    componentDidUpdate(prevProps: DayGridLayoutProps<unknown, unknown>): void;
    componentWillUnmount(): void;
    private timeScrollResponder;
}

interface DayGridRowProps {
    dateProfile: DateProfile;
    todayRange: DateRange;
    cells: DayTableCell[];
    showDayNumbers: boolean;
    showWeekNumbers?: boolean;
    forPrint: boolean;
    cellGroup?: boolean;
    className?: string;
    forceVSpacing?: boolean;
    compact?: boolean;
    fgEventSegs: TableSeg[];
    bgEventSegs: TableSeg[];
    businessHourSegs: TableSeg[];
    dateSelectionSegs: TableSeg[];
    eventDrag: EventSegUiInteractionState | null;
    eventResize: EventSegUiInteractionState | null;
    eventSelection: string;
    dayMaxEvents: boolean | number;
    dayMaxEventRows: boolean | number;
    colWidth?: number;
    minHeight?: number | string;
    rootElRef?: Ref<HTMLElement>;
    heightRef?: Ref<number>;
    innerHeightRef?: Ref<number>;
}
interface DayGridRowState {
    innerHeight?: number;
    headerHeight?: number;
    segHeightRev?: string;
}
declare const COMPACT_CELL_WIDTH = 80;
declare class DayGridRow extends BaseComponent<DayGridRowProps, DayGridRowState> {
    private rootEl;
    private cellInnerHeightRefMap;
    private cellHeaderHeightRefMap;
    private segHeightRefMap;
    private disconnectHeight?;
    render(): createElement.JSX.Element;
    renderFgSegs(segs: TableSeg[], segTops: {
        [segStartId: string]: number;
    }, todayRange: DateRange, forcedInvisibleMap: {
        [instanceId: string]: any;
    }, isDragging?: boolean, isResizing?: boolean, isDateSelecting?: boolean): VNode[];
    renderFillSegs(segs: TableSeg[], fillType: string): VNode;
    handleRootEl: (rootEl: HTMLElement) => void;
    componentDidMount(): void;
    componentWillUnmount(): void;
    private handleHeaderHeights;
    private handleInnerHeights;
    private handleSegHeights;
    getMirrorSegs(): TableSeg[];
    getHighlightSegs(): TableSeg[];
}

interface DayGridRowsProps {
    dateProfile: DateProfile;
    todayRange: DateRange;
    cellRows: DayTableCell[][];
    forPrint: boolean;
    isHitComboAllowed?: (hit0: Hit, hit1: Hit) => boolean;
    fgEventSegs: TableSeg[];
    bgEventSegs: TableSeg[];
    businessHourSegs: TableSeg[];
    dateSelectionSegs: TableSeg[];
    eventDrag: EventSegUiInteractionState | null;
    eventResize: EventSegUiInteractionState | null;
    eventSelection: string;
    colWidth?: number;
    width?: number | string;
    rowHeightRefMap?: RefMap<string, number>;
}
interface DayGridRowsState {
    width?: number;
}
declare class DayGridRows extends DateComponent<DayGridRowsProps, DayGridRowsState> {
    private rootEl;
    private splitBusinessHourSegs;
    private splitBgEventSegs;
    private splitFgEventSegs;
    private splitDateSelectionSegs;
    private splitEventDrag;
    private splitEventResize;
    private rowHeightRefMap;
    private unwatchWidth?;
    render(): createElement.JSX.Element;
    handleRootEl: (rootEl: HTMLDivElement) => void;
    componentDidMount(): void;
    componentWillUnmount(): void;
    queryHit(positionLeft: number, positionTop: number, elWidth: number): Hit;
}

declare function buildDayTableModel(dateProfile: DateProfile, dateProfileGenerator: DateProfileGenerator): DayTableModel;
declare function computeColWidth(colCnt: number, colMinWidth: number, viewportWidth: number | undefined): [
    canvasWidth: number | undefined,
    colWidth: number | undefined
];
type DateHeaderCellObj = {
    date: DateMarker;
    colSpan: number;
};
type DayOfWeekHeaderCellObj = {
    dow: number;
    colSpan: number;
};
type HeaderCellObj = DateHeaderCellObj | DayOfWeekHeaderCellObj;
declare function computeColFromPosition(positionLeft: number, elWidth: number, colWidth: number | undefined, colCnt: number, isRtl: boolean): {
    col: number;
    left: number;
    right: number;
};
declare function getRowEl(rootEl: HTMLElement, row: number): HTMLElement;
declare function getCellEl(rowEl: HTMLElement, col: number): HTMLElement;

export { COMPACT_CELL_WIDTH, DateHeaderCell, DateHeaderCellObj, DayGridLayout, DayGridLayoutProps, DayGridRow, DayGridRowProps, DayGridRows, DayGridView, DayOfWeekHeaderCell, DayOfWeekHeaderCellObj, DayTableSlicer, HeaderCellObj, HeaderRow, HeaderRowAdvanced, TableDateProfileGenerator, TableSeg, buildDayTableModel, buildDayTableRenderRange, computeColFromPosition, computeColWidth, createDayHeaderFormatter, getCellEl, getRowEl };
